import { requireAuthentication, t } from "./base"
import { getNumberCardsSum } from "../../shared/game/state"
import { z } from "zod"
import { getGameByLobbyCode } from "../game"

const gameProcedure = t.procedure
	.use(requireAuthentication)
	.input(z.object({
		lobbyCode: z.string()
	}))
	.use(t.middleware(async ({ input, ctx, next }) => {
		const game = getGameByLobbyCode((input as { lobbyCode: string }).lobbyCode)

		if (game === null) throw new Error("The game does not exist or is not active")
		return await next({
			ctx: {
				game
			}
		})
	}))

export const gameRouter = t.router({
	start: gameProcedure
		.mutation(async ({ ctx }) => {
			if (ctx.game.state.phase !== "pre-start") throw new Error(`Cannot start the game in this phase: ${ctx.game.state.phase}`)
			if (ctx.game.state.players.findIndex(p => p.id === ctx.user.id) !== 0) throw new Error("Only the creator can start the game")
			ctx.game.start()
		}),

	hit: gameProcedure
		.mutation(async ({ ctx }) => {
			if (ctx.game.state.activePlayerId !== ctx.user.id) throw new Error("It is not the player’s turn")
			if (getNumberCardsSum(ctx.game.state.players.find(p => p.id === ctx.user.id)!.numberCards) > ctx.game.state.targetSum)
				throw new Error("The player cannot hit when bust")

			await ctx.game.hit()
		}),

	stay: gameProcedure
		.mutation(async ({ ctx }) => {
			if (ctx.game.state.activePlayerId !== ctx.user.id) throw new Error("It is not the player’s turn")
			await ctx.game.stay()
		})
})